/*
 * ClassWrapper.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;
import java.util.Vector;

/**
 * 
 * @author Carl
 */
public class ClassWrapper {

	private static final String INDENT = "    ";

	private static final String INDENT2 = INDENT + INDENT;

	private static final String INDENT3 = INDENT + INDENT2;

	private static final String INDENT4 = INDENT + INDENT3;

	private Class<?> clazz;

	private String packageName;

	private String clazzName;

	private Methods methods;

	private Vector<ConstructorWrapper> constructors;

	private boolean isCldc11;

	private String getHeaderComment() {
		StringBuffer sb = new StringBuffer();
		sb.append("/*\n");
		sb.append(" * This source code was generated by HammockMaker.\n");
		sb.append(" * It should be used with the Hammock libraries for the\n");
		sb.append(" * CLDC 1." + (isCldc11 ? "1" : "0") + " configuration.\n");
		sb.append(" */\n");
		return sb.toString();
	}

	private String getPackageDeclaration() {
		if ((this.packageName == null) || (this.packageName.equals(""))) {
			return "";
		}

		return "package " + this.packageName + ";\n\n";
	}

	private String getImports() {
		return "import com.hammingweight.hammock.*;\n\n";
	}

	private String getClassDeclaration() {
		String s = "public class " + this.clazzName + " extends ";
		if (clazz.isInterface()) {
			s += "AMockObject implements " + this.clazz.getCanonicalName();
		} else {
			s += this.clazz.getCanonicalName() + " implements IMockObject";
		}

      if (!this.isCldc11) {
         s += ", IClassDefinitions ";
      }
      s += " ";
      
		return s;
	}

	public String getMockFieldDeclarations() {
		if (clazz.isInterface()) {
			return "";
		}

		return "\n" + INDENT + "private IInvocationHandler handler;\n\n";
	}

	private String getMockMethodDeclarations() {
		if (clazz.isInterface()) {
			return "";
		}

		String s = INDENT + "// Methods defined in IMockObject.\n";
		s += INDENT
				+ "public final void setInvocationHandler(IInvocationHandler handler) {\n";
		s += INDENT2 + "if (handler == null) {\n";
		s += INDENT3 + "throw new NullPointerException();\n";
		s += INDENT2 + "}\n\n";
		s += INDENT2 + "this.handler = handler;\n";
		s += INDENT + "}\n\n";
		s += INDENT
				+ "public final IInvocationHandler getInvocationHandler() {\n";
		s += INDENT2 + "if (this.handler == null) {\n";
		s += INDENT3
				+ "setInvocationHandler(new Hamspy());\n";
		s += INDENT2 + "}\n";
		s += INDENT2 + "return this.handler;\n";
		s += INDENT + "}\n\n";

		return s;
	}

	private static String getAbstractBody(MethodWrapper methodWrapper) {
		String s = INDENT3 + "MethodInvocation mi = new MethodInvocation(";
		s += methodWrapper.getMethodIdentifier() + ", this, args);\n";
		s += INDENT3 + "getInvocationHandler().invoke(mi);\n";
		s += INDENT3;
		if (!methodWrapper.isReturnTypeVoid()) {
			s += "Object retVal = ";
		}
		s += "mi.getReturnValue();\n";
		if (!methodWrapper.isReturnTypeVoid()) {
			if (methodWrapper.isReturnTypePrimitive()) {
				s += INDENT3 + "AMockObject.assertReturnNotNull("
						+ methodWrapper.getMethodIdentifier() + ", retVal);\n";
				s += INDENT3 + "return "
						+ methodWrapper.getPrimitiveReturnTypeCast() + ";\n";
			} else {
				s += INDENT3 + "return (" + methodWrapper.getReturnType()
						+ ")retVal;\n";
			}
		}
		return s;
	}

	private static String getConcreteBody(MethodWrapper methodWrapper) {
		String s = INDENT3 + "MethodInvocation mi = new MethodInvocation(";
		s += methodWrapper.getMethodIdentifier() + ", this, args);\n";
		s += INDENT3 + "getInvocationHandler().invoke(mi);\n";
		s += INDENT3 + "if (mi.isEvaluated()) {\n";
		s += INDENT4;
		if (!methodWrapper.isReturnTypeVoid()) {
			s += "Object retVal = ";
		}
		s += "mi.getReturnValue();\n";
		if (methodWrapper.isReturnTypeVoid()) {
			s += INDENT4 + "return;\n";
		} else {
			if (methodWrapper.isReturnTypePrimitive()) {
				s += INDENT4 + "AMockObject.assertReturnNotNull("
						+ methodWrapper.getMethodIdentifier() + ", retVal);\n";
				s += INDENT4 + "return "
						+ methodWrapper.getPrimitiveReturnTypeCast() + ";\n";
			} else {
				s += INDENT4 + "return (" + methodWrapper.getReturnType()
						+ ")retVal;\n";
			}
		}
		s += INDENT3 + "}\n";
		return s;
	}

	private static String getSuperMethod(MethodWrapper methodWrapper) {
		String s = INDENT2;
		if (!methodWrapper.isReturnTypeVoid()) {
			s += "return ";
		}
		s += "super.";
		s += methodWrapper.getName();
		s += "(";
		Arguments args = methodWrapper.getArguments();
		for (int i = 0; i < args.getNumberOfArgs(); i++) {
			s += "arg" + Integer.toString(i);
			if (i != args.getNumberOfArgs() - 1) {
				s += ", ";
			}
		}
		s += ");\n";
		return s;
	}

	private String getMethodBody(MethodWrapper methodWrapper) {
		if (methodWrapper.isFinal()) {
			return "";
		}
		
		String s = INDENT + "public static final MockMethod "
				+ methodWrapper.getMethodIdentifier();
		s += " = new MockMethod(";
		if (this.isCldc11) {
			s += "\n";
			s += INDENT2 + this.clazzName + ".class, \n" + INDENT2 + "\""
					+ methodWrapper.getMethodIdentifier() + "\",\n" + INDENT2
					+ "new Class[]{";
			int numArgs = methodWrapper.getArguments().getNumberOfArgs();
			for (int i = 0; i < numArgs; i++) {
				s += methodWrapper.getArguments().getObjectClass(i);
				if (i != (numArgs - 1))
					s += ", ";
			}
			s += "},\n";
			s += INDENT2 + "new Class[]{";
			Exceptions exceps = methodWrapper.getExceptions();
			int numExceptionTypes = exceps.getNumberOfExceptions();
			for (int i = 0; i < numExceptionTypes; i++) {
				s += exceps.getExceptionNumber(i) + ".class";
				if (i != (numExceptionTypes - 1)) {
					s += ", ";
				}
			}
			s += "},\n";
			if (methodWrapper.isReturnTypeVoid()) {
				s += INDENT2 + "null,\n";
			} else {
				s += INDENT2 + methodWrapper.getObjectReturnType() + ",\n";
			}
			s += INDENT2 + methodWrapper.isAbstract() + ");\n";
		} else {
			s += "\"" + methodWrapper.getMethodIdentifier() + "\", ";
			s += Integer.toString(methodWrapper.getArguments()
					.getNumberOfArgs())
					+ ", ";
			if (methodWrapper.isReturnTypeVoid()) {
				s += "null, ";
			} else {
				s += methodWrapper.getReturnClass() + ", ";
			}
			s += Boolean.toString(methodWrapper.isAbstract()) + ");\n";
		}

		s += INDENT + methodWrapper.getDeclaration();
		s += " {\n";

		s += INDENT2 + "try {\n";
		Arguments args = methodWrapper.getArguments();
		s += INDENT3 + "Object[] args = new Object[" + args.getNumberOfArgs()
				+ "];\n";
		for (int i = 0; i < args.getNumberOfArgs(); i++) {
			s += INDENT3 + "args[" + Integer.toString(i) + "] = ";
			s += args.getObjectArg(i) + ";\n";
		}

		// handle abstract and non-abstract bodies
		if (methodWrapper.isAbstract()) {
			s += getAbstractBody(methodWrapper);
		} else {
			s += getConcreteBody(methodWrapper);
		}

		s += INDENT2 + "} catch (Throwable t) {\n";
		s += INDENT3;
		s += "if (t instanceof java.lang.Error) { throw (java.lang.Error)t; }\n";
		s += INDENT3;
		s += "if (t instanceof java.lang.RuntimeException) { throw (java.lang.RuntimeException)t; }\n";
		Exceptions excepts = methodWrapper.getExceptions();
		for (int i = 0; i < excepts.getNumberOfExceptions(); i++) {
			String ex = excepts.getExceptionNumber(i);
			s += INDENT3 + "if (t instanceof " + ex + ") { throw (" + ex
					+ ")t; }\n";
		}
		s += INDENT3 + "throw new HammockException(t);\n";
		s += INDENT2 + "}\n";

		if (!methodWrapper.isAbstract()) {
			s += getSuperMethod(methodWrapper);
		}
		return s + INDENT + "}\n\n";
	}

	private static String getConstructorBody(
			ConstructorWrapper constructorWrapper) {
		String s = INDENT + constructorWrapper.getDeclaration();
		s += " {\n";
		s += INDENT2 + "super(";
		Arguments args = constructorWrapper.getArguments();
		for (int i = 0; i < args.getNumberOfArgs(); i++) {
			s += Arguments.VAR_ARG_NAME_PREFIX + Integer.toString(i);
			if (i != args.getNumberOfArgs() - 1) {
				s += ", ";
			}
		}
		s += ");\n";
		return s + INDENT + "}\n\n";
	}

   private static String getAugmentedConstructorBody(
			ConstructorWrapper constructorWrapper) {
		String s = INDENT + constructorWrapper.getAugmentedDeclaration();
		s += " {\n";
		s += INDENT2 + "super(";
		Arguments args = constructorWrapper.getArguments();
		for (int i = 0; i < args.getNumberOfArgs(); i++) {
			s += Arguments.VAR_ARG_NAME_PREFIX + Integer.toString(i);
			if (i != args.getNumberOfArgs() - 1) {
				s += ", ";
			}
		}
		s += ");\n";
		s += INDENT2 + "setInvocationHandler(handler);\n";
		return s + INDENT + "}\n\n";
	}
	
	private String getDefaultConstructors() {
      String s = INDENT + "public " + this.clazzName + "() {\n";
      s += INDENT2 + "super();\n";
      s += INDENT + "}\n\n";
      s += INDENT + "public " + this.clazzName + "(IInvocationHandler handler) {\n";
      s += INDENT2 + "super();\n";
      s += INDENT2 + "setInvocationHandler(handler);\n";
		return s + INDENT + "}\n\n";
	}
	
	private String getConstructorDeclarations() {
		String s = "";

      s += INDENT + "// Constructors.\n";
		for (ConstructorWrapper wrapper : this.constructors) {
			s += getConstructorBody(wrapper);
			s += getAugmentedConstructorBody(wrapper);
		}
		
		if (this.constructors.size() == 0) {
         s += getDefaultConstructors();
		}

		return s;
	}

	private String getMethodDeclarations() {
		String s = "";

		for (MethodWrapper wrapper : this.methods) {
			s += getMethodBody(wrapper);
		}

		return s;
	}

	/** Creates a new instance of ClassWrapper */
	public ClassWrapper(Class<?> clazz, String packageName, String clazzName,
			boolean isCldc11) throws Exception {
		this.isCldc11 = isCldc11;
		this.clazz = clazz;
		this.packageName = packageName;
		this.clazzName = clazzName;
		this.methods = new Methods(clazz);
		this.constructors = new Vector<ConstructorWrapper>();

		Constructor<?>[] cons = clazz.getConstructors();
		for (Constructor<?> constructor : cons) {
			if (Modifier.isFinal(constructor.getModifiers())) {
				continue;
			}
			constructors
					.add(new ConstructorWrapper(this.clazzName, constructor));
		}

		cons = clazz.getDeclaredConstructors();
		for (Constructor<?> constructor : cons) {
			if (!Modifier.isProtected(constructor.getModifiers())) {
				continue;
			}
			constructors
					.add(new ConstructorWrapper(this.clazzName, constructor));
		}
	}

	public static String classToString(Class<?> clazz) {
		while (clazz.isArray()) {
			clazz = clazz.getComponentType();
			return classToString(clazz) + "[]";
		}

		String name = clazz.getCanonicalName();
		if (name != null) {
			return name;
		} else {
			return clazz.getName();
		}
	}

	public static String getWrapperForPrimitiveType(Class<?> clazz) {
		String type = null;
		if (clazz == boolean.class) {
			type = "Boolean";
		} else if (clazz == byte.class) {
			type = "Byte";
		} else if (clazz == char.class) {
			type = "Character";
		} else if (clazz == short.class) {
			type = "Short";
		} else if (clazz == int.class) {
			type = "Integer";
		} else if (clazz == long.class) {
			type = "Long";
		} else if (clazz == float.class) {
			type = "Float";
		} else if (clazz == double.class) {
			type = "Double";
		}

		return type;
	}
	
	public String toString() {
		String s = getHeaderComment();
		s += getPackageDeclaration();
		s += getImports();
		s += getClassDeclaration();
		s += "{\n";
		s += getMockFieldDeclarations();
		s += getMockMethodDeclarations();

		s += INDENT + "// Overridden methods.\n";
		s += getMethodDeclarations();
		s += getConstructorDeclarations();
		s += "}\n";
		return s;
	}
}
