/*
 * Exceptions.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

import java.util.Vector;

/**
 *
 * @author Carl
 */
public class Exceptions {
   
   private Vector<Class<?>> exceptionTypes;
   
   public Exceptions(Class<?>[] exceptionClasses) {
      exceptionTypes = new Vector<Class<?>>();
      for (Class<?> clazz : exceptionClasses) {
         if (java.lang.RuntimeException.class.isAssignableFrom(clazz)) {
            continue;
         }
         if (java.lang.Error.class.isAssignableFrom(clazz)) {
        	 continue;
         }
         this.exceptionTypes.add(clazz);
      }
   }
   
   public String getThrowsClause() {
      if (exceptionTypes.size() == 0) {
         return "";
      }
      
      String s = "throws ";
      for (int i = 0; i < this.exceptionTypes.size(); i++) {
         s += ClassWrapper.classToString(this.exceptionTypes.elementAt(i));
         if (i != this.exceptionTypes.size() - 1) {
            s += ", ";
         }
      }
      
      return s;
   }
   
   public int getNumberOfExceptions() {
      return this.exceptionTypes.size();
   }
   
   public String getExceptionNumber(int i) {
      return ClassWrapper.classToString(this.exceptionTypes.elementAt(i));
   }
}
