package com.hammingweight.hammock;

import junit.framework.TestCase;

public class DefaultArgumentMatcherTest extends TestCase {
   private static final String STRING_OBJ1 = new String("Foo");
   private static final String STRING_OBJ2 = new String("Foo");
   private static final String STRING_OBJ3 = new String("Bar");
   private static final Integer INTEGER_OBJ1 = new Integer(1);
   private static final Integer INTEGER_OBJ2 = new Integer(1);
   private static final Integer INTEGER_OBJ3 = new Integer(0);
   
   // The default argument matcher regards arrays as equal if they contain elements that are equal.
   private static final boolean[] BOOL_ARRAY1 = new boolean[]{false, false, true};
   private static final boolean[] BOOL_ARRAY2 = new boolean[]{false, false, true};
   private static final boolean[] BOOL_ARRAY3 = new boolean[]{false, false, false};
   private static final boolean[][] BOOL2_ARRAY1 = new boolean[][]{{false, false}, {true}};
   private static final boolean[][] BOOL2_ARRAY2 = new boolean[][]{{false, false}, {true}};
   private static final boolean[][] BOOL2_ARRAY3 = new boolean[][]{{false, false}, {false}};
   
   private static final byte[] BYTE_ARRAY1 = new byte[]{(byte)1, (byte)2, (byte)3};
   private static final byte[] BYTE_ARRAY2 = new byte[]{(byte)1, (byte)2, (byte)3};
   private static final byte[] BYTE_ARRAY3 = new byte[]{(byte)0, (byte)2, (byte)3};
   private static final byte[][] BYTE2_ARRAY1 = new byte[][]{{(byte)1, (byte)2}, {(byte)3}};
   private static final byte[][] BYTE2_ARRAY2 = new byte[][]{{(byte)1, (byte)2}, {(byte)3}};
   private static final byte[][] BYTE2_ARRAY3 = new byte[][]{{(byte)0, (byte)2}, {(byte)3}};
   
   private static final short[] SHORT_ARRAY1 = new short[]{(short)1, (short)2, (short)3};
   private static final short[] SHORT_ARRAY2 = new short[]{(short)1, (short)2, (short)3};
   private static final short[] SHORT_ARRAY3 = new short[]{(short)0, (short)2, (short)3};
   private static final short[][] SHORT2_ARRAY1 = new short[][]{{(short)1, (short)2}, {(short)3}};
   private static final short[][] SHORT2_ARRAY2 = new short[][]{{(short)1, (short)2}, {(short)3}};
   private static final short[][] SHORT2_ARRAY3 = new short[][]{{(short)1, (short)2}, {(short)3, (short)4}};
   
   private static final int[] INT_ARRAY1 = new int[]{1, 2, 3};
   private static final int[] INT_ARRAY2 = new int[]{1, 2, 3};
   private static final int[] INT_ARRAY3 = new int[]{1, 2, 4};
   private static final int[][][] INT3_ARRAY1 = new int[][][]{{{1,2},{3}},{{4}}};
   private static final int[][][] INT3_ARRAY2 = new int[][][]{{{1,2},{3}},{{4}}};
   private static final int[][][] INT3_ARRAY3 = new int[][][]{{{1,2},{3}},{{4,5}}};
   
   private static final char[] CHAR_ARRAY1 = new char[]{'c', 'h', 'a', 'r'};
   private static final char[] CHAR_ARRAY2 = new char[]{'c', 'h', 'a', 'r'};
   private static final char[] CHAR_ARRAY3 = new char[]{'c', 'h', 'a', 'r', 's'};
   
   private static final long[] LONG_ARRAY1 = new long[]{0x7FFFFFFFFFL, 1L, -0x7FFFFFFFFFL};
   private static final long[] LONG_ARRAY2 = new long[]{0x7FFFFFFFFFL, 1L, -0x7FFFFFFFFFL};
   private static final long[] LONG_ARRAY3 = new long[]{0x7FFFFFFFFFL, 1L, 0x7FFFFFFFFFL};
   
   private static final String[] STRING_ARRAY1 = new String[]{STRING_OBJ1, STRING_OBJ2};
   private static final String[] STRING_ARRAY2 = new String[]{STRING_OBJ1, STRING_OBJ2};
   private static final String[] STRING_ARRAY3 = new String[]{STRING_OBJ1, STRING_OBJ3};
   
   public static final Object[] OBJECT_ARRAY1 = new Object[]{INTEGER_OBJ1, STRING_ARRAY1, INT3_ARRAY1};
   public static final Object[] OBJECT_ARRAY1B = new Object[]{INTEGER_OBJ1, STRING_ARRAY1, INT3_ARRAY2};
   public static final Object[] OBJECT_ARRAY2 = new Object[]{INTEGER_OBJ2, STRING_ARRAY2, INT3_ARRAY2};
   public static final Object[] OBJECT_ARRAY113 = new Object[]{INTEGER_OBJ1, STRING_ARRAY1, INT3_ARRAY3};
   public Object[] OBJECT_ARRAY131 = new Object[]{INTEGER_OBJ1, STRING_ARRAY3, INT3_ARRAY1};
   public Object[] OBJECT_ARRAY311 = new Object[]{INTEGER_OBJ3, STRING_ARRAY1, INT3_ARRAY1};
   
   // The OUT.
   private DefaultArgumentMatcher dam = new DefaultArgumentMatcher();
   
   public void testBasicComparisons() {
      assertTrue(this.dam.areArgumentsEqual(STRING_OBJ1, STRING_OBJ2));
      assertTrue(this.dam.areArgumentsEqual(INTEGER_OBJ1, INTEGER_OBJ2));

      assertFalse(this.dam.areArgumentsEqual(STRING_OBJ1, STRING_OBJ3));
      assertFalse(this.dam.areArgumentsEqual(INTEGER_OBJ1, INTEGER_OBJ3));
   }
   
   public void testBooleanArrays() {
	   // The contents of 1-D and 2-D arrays 1 and 2 are equal.
      assertTrue(this.dam.areArgumentsEqual(BOOL_ARRAY1, BOOL_ARRAY2));
      assertTrue(this.dam.areArgumentsEqual(BOOL2_ARRAY1, BOOL2_ARRAY2));

      // The contents of arrays 1 and 3 and of arrays 2 and 3 are 
      assertFalse(this.dam.areArgumentsEqual(BOOL_ARRAY1, BOOL_ARRAY3));
      assertFalse(this.dam.areArgumentsEqual(BOOL2_ARRAY1, BOOL2_ARRAY3));
      
      // A 1-D and 2-D array are definitely different.
      assertFalse(this.dam.areArgumentsEqual(BOOL_ARRAY1, BOOL2_ARRAY1));
   }
   
   public void testByteArrays() {
      assertTrue(this.dam.areArgumentsEqual(BYTE_ARRAY1, BYTE_ARRAY2));
      assertTrue(this.dam.areArgumentsEqual(BYTE2_ARRAY1, BYTE2_ARRAY2));

      assertFalse(this.dam.areArgumentsEqual(BYTE_ARRAY1, BYTE_ARRAY3));
      assertFalse(this.dam.areArgumentsEqual(BYTE2_ARRAY1, BYTE2_ARRAY3));

      assertFalse(this.dam.areArgumentsEqual(BYTE_ARRAY1, BYTE2_ARRAY1));
   }
   
   public void testShortArrays() {
      assertTrue(this.dam.areArgumentsEqual(SHORT_ARRAY1, SHORT_ARRAY2));
      assertTrue(this.dam.areArgumentsEqual(SHORT2_ARRAY1, SHORT2_ARRAY2));

      assertFalse(this.dam.areArgumentsEqual(SHORT_ARRAY1, SHORT_ARRAY3));
      assertFalse(this.dam.areArgumentsEqual(SHORT2_ARRAY1, SHORT2_ARRAY3));

      assertFalse(this.dam.areArgumentsEqual(SHORT_ARRAY1, SHORT2_ARRAY1));
      assertFalse(this.dam.areArgumentsEqual(SHORT2_ARRAY3, SHORT2_ARRAY2));
      
      // Comparing a short and byte array should fail.
      assertFalse(this.dam.areArgumentsEqual(SHORT_ARRAY1, BYTE_ARRAY1));
   }
   
   public void testIntArrays() {
	   // Compare equal 1- and 3-D arrays.
      assertTrue(this.dam.areArgumentsEqual(INT_ARRAY1, INT_ARRAY2));
      assertTrue(this.dam.areArgumentsEqual(INT3_ARRAY1, INT3_ARRAY2));
      
      // Compare different 1- and 3-D arrays.
      assertFalse(this.dam.areArgumentsEqual(INT_ARRAY1, INT_ARRAY3));
      assertFalse(this.dam.areArgumentsEqual(INT3_ARRAY1, INT3_ARRAY3));

      // Compare a 1-D and a 3-D array.
      assertFalse(this.dam.areArgumentsEqual(INT_ARRAY1, INT3_ARRAY1));
   }
   
   public void testCharArrays() {
      assertTrue(this.dam.areArgumentsEqual(CHAR_ARRAY1, CHAR_ARRAY2));
      assertTrue(this.dam.areArgumentsEqual(CHAR_ARRAY2, CHAR_ARRAY1));

      assertFalse(this.dam.areArgumentsEqual(CHAR_ARRAY1, CHAR_ARRAY3));
      assertFalse(this.dam.areArgumentsEqual(CHAR_ARRAY2, CHAR_ARRAY3));
      assertFalse(this.dam.areArgumentsEqual(CHAR_ARRAY3, CHAR_ARRAY1));
      assertFalse(this.dam.areArgumentsEqual(CHAR_ARRAY3, CHAR_ARRAY2));
   }
   
   public void testLongArrays() {
      assertTrue(this.dam.areArgumentsEqual(LONG_ARRAY1, LONG_ARRAY2));
      
      assertFalse(this.dam.areArgumentsEqual(LONG_ARRAY1, LONG_ARRAY3));
   }
   
   public void testStringArrays() {
      assertTrue(this.dam.areArgumentsEqual(STRING_ARRAY1, STRING_ARRAY2));
      
      assertFalse(this.dam.areArgumentsEqual(STRING_ARRAY1, STRING_ARRAY3));
   }
   
   public void testObjectArrays() {
      assertTrue(this.dam.areArgumentsEqual(OBJECT_ARRAY1, OBJECT_ARRAY2));
      assertTrue(this.dam.areArgumentsEqual(OBJECT_ARRAY1, OBJECT_ARRAY1B));
      
      assertFalse(this.dam.areArgumentsEqual(OBJECT_ARRAY1, OBJECT_ARRAY113));
      assertFalse(this.dam.areArgumentsEqual(OBJECT_ARRAY1, OBJECT_ARRAY131));
      assertFalse(this.dam.areArgumentsEqual(OBJECT_ARRAY1, OBJECT_ARRAY311));
      assertFalse(this.dam.areArgumentsEqual(OBJECT_ARRAY113, OBJECT_ARRAY131));
      assertFalse(this.dam.areArgumentsEqual(OBJECT_ARRAY131, OBJECT_ARRAY311));
   }
}
